import os
import uuid

from django.conf import settings
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_http_methods
from .configs import VditorConfig


VDITOR_CONFIGS = VditorConfig('default')


@require_http_methods(['POST'])
@method_decorator(csrf_exempt)
def VditorImagesUploadView(request):
    VditorImagesUpload = request.FILES.get('file[]', None)
    VditorImagesNameList = VditorImagesUpload.name.split('.')
    VditorImagesName = '.'.join(VditorImagesNameList)
    VditorImagesUploadPath = os.path.join(settings.MEDIA_ROOT)
    VditorImagesNameFull = '%s_%s' % (uuid.uuid4(), VditorImagesName)
    if not VditorImagesNameList:
        print('No picture')
    else:
        if not os.path.exists(VditorImagesUploadPath):
            try:
                os.makedirs(VditorImagesUploadPath)
            except Exception as err:
                print("upload failed：%s" % str(err))
        else:
            with open(os.path.join(VditorImagesUploadPath, VditorImagesNameFull), 'wb+') as file:
                for chunk in VditorImagesUpload.chunks():
                    file.write(chunk)
            return JsonResponse(
                {
                    "msg": "Success!",
                    "code": 0,
                    "data": {
                    "errFiles": [],
                    "succMap": {
                        VditorImagesName: os.path.join(settings.MEDIA_URL, VditorImagesNameFull),
                        }
                    }
                }
            )



from django import forms
from django.forms.utils import flatatt
from django.forms.widgets import get_default_renderer
from django.utils.encoding import force_text
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from .configs import VditorConfig

class VditorWidget(forms.Textarea):
    def __init__(self, config_name = 'default', *args, **kwargs):
        super(VditorWidget, self).__init__(*args, **kwargs)
        self.config = VditorConfig(config_name)

    def render(self, value, name, attrs=None, renderer=None):
        if renderer is None:
            renderer = get_default_renderer()
        if value is None:
            value = ""
        final_attrs = self.build_attrs(self.attrs, attrs, name=name)
        
        return mark_safe(
            renderer.render(
                'widget.html',
                {
                    'final_attrs': flatatt(final_attrs),
                    'value': conditional_escape(force_text(value)),
                    'id': final_attrs['id'],
                    'config': self.config,
                }
                )
            )

    def build_attrs(self, base_attrs, extra_attrs=None, **kwargs):
        attrs = dict(base_attrs, **kwargs)
        if extra_attrs:
            attrs.update(extra_attrs)
        return attrs

    def _get_media(self):
        return forms.Media(
            css={
                "all": ("dist/index.css",)
            },
            js=(
                "dist/index.min.js",
            ))
    media = property(_get_media)


from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

#There are currently fewer options, more options will be added in the future
DEFAULT_CONFIG = {
    "width": "%90",
    "height": 360,
    "preview_theme": "light",
    "typewriterMode": "True",
    "mode": "ir",
    "debugger": "false",
    "value": "",
    "theme": "classic",
    "icon": "ant",
    "outline": "false",
}

if settings.LANGUAGE_CODE == "zh-Hans":
    DEFAULT_CONFIG["lang"] = "zh_CN"
elif settings.LANGUAGE_CODE == "ja-jp":
    DEFAULT_CONFIG["lang"] = "ja_JP"
elif settings.LANGUAGE_CODE == "ko-kr":
    DEFAULT_CONFIG["lang"] = "ko_KR"
else:
    DEFAULT_CONFIG["lang"] = "en_US"

class VditorConfig(dict):
    def __init__(self, config_name = "default"):
        self.update(DEFAULT_CONFIG)
        self.set_configs(config_name)

    def set_configs(self, config_name = "default"):
        configs = getattr(settings, "VDITOR_CONFIGS", None)
        if configs:
            if isinstance(configs, dict):
                if config_name in configs:
                    config = configs[config_name]
                    if not isinstance(config, dict):
                        raise ImproperlyConfigured('VDITOR_CONFIGS["%s"] \
                                        setting must be a dictionary type.' %config_name)
                    self.update(config)
                else:
                    raise ImproperlyConfigured("No configuration named '%s' \
                                    found in your VDITOR_CONFIGS setting." %config_name)
            else:
                raise ImproperlyConfigured('VDITOR_CONFIGS setting must be a\
                                dictionary type.')
